﻿using System;

namespace ExtensibleObjectProperties
{
	/// <summary>
	/// A property is composed of an attribute and a value assigned to this attribute.
	/// (An attribute is a name and a data type.
	/// It ensures that the value assigned to the property if of the correct data type.)
	/// </summary>
	public class xProperty
	{

		xAttribute			m_attribute				= null;
		protected object	m_value					= null;

		public xProperty()
		{
		}

		public xProperty(xAttribute attrib, object obj_value)
		{
			m_attribute		= attrib;
			Value			= obj_value;
		}

		public xAttribute Attribute
		{
			get { return m_attribute; }
			set { m_attribute = value; }
		}

		public Type Property_type
		{
			get { return (m_attribute == null) ?(null) :(m_attribute.Data_type); }
		}


		public object Value
		{
			get { return m_value; }
			set
			{
				if( CanConvertValue( value))
					m_value = value;
			}
		}

		public bool CanConvertValue(object new_value)
		{
			if (m_attribute == null || new_value == null)
				return true;

			return m_attribute.CanConvertValue( new_value);
		}


		public byte[] ValueAsByteArray
		{
			get
			{
				if (m_value is byte[])
					return (byte[])m_value;
				return new byte[1];
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}



		public bool ValueAsBoolean
		{
			get
			{
				if( m_value is bool)
					return (bool) m_value;
				return false;
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}

		public int ValueAsInt
		{
			get
			{
				if (m_value is int)
					return (int)m_value;
				return 0;
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}

		public long ValueAsLong
		{
			get
			{
				if (m_value is long)
					return (long)m_value;
				return 0;
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}

		public double ValueAsDouble
		{
			get
			{
				if (m_value is double)
					return (double)m_value;
				return 0L;
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}

		public string ValueAsString
		{
			get
			{
				if (m_value is string)
					return m_value.ToString();
				return "";
			}
			set
			{
				if (CanConvertValue(value))
					m_value = value;
			}
		}

	}
}
